@@ -1,9 +1,8 @@
-from _transform import *
-from _reshape import *
-from _indexing import *
+from transform import *
+from reshape import *
+from indexing import *
 from interpolation import *
 from missingvalues import *
-#from _operation import *
 
 from axes import Axis, Axes, GroupedAxis
 from dimarraycls import DimArray, array, array_kw, empty, zeros, ones, nans, empty_like, zeros_like, ones_like, nans_like, from_arrays, from_pandas, join
