@@ -10,6 +10,8 @@ from tools import is_array1d_equiv
 __all__ = ["Axis","Axes", "is_regular"]
 
 def is_regular(values):
+    """ test if numeric, monotonically increasing and constant step
+    """
     if values.dtype is np.dtype('O'): 
 	regular = False
 
@@ -20,6 +22,20 @@ def is_regular(values):
 
     return regular
 
+def is_monotonic(values):
+    """ test is monotonically increasing or decreasing
+    """
+    if values.size < 2:
+	monotonic = True
+
+    else:
+	#increasing = np.diff(values) > 0 
+	increasing = values[1:] >= values[:-1] 
+	monotonic = np.all(increasing) or np.all(values[1:] <= values[:-1])
+
+    return monotonic
+
+
 def _convert_dtype(dtype):
     """ convert numpy type in a python type
     """
@@ -52,7 +68,7 @@ class Axis(object):
     """
     _metadata = MetadataDesc(exclude = ["values", "name", "weights", "modulo"]) # variables which are NOT metadata
 
-    def __init__(self, values, name="", weights=None, modulo=None, dtype=None, **kwargs):
+    def __init__(self, values, name="", weights=None, modulo=None, dtype=None, _monotonic=None, **kwargs):
 	""" 
 	"""
 	if not name:
@@ -85,6 +101,7 @@ class Axis(object):
 	self.name = name 
 	self.weights = weights 
 	self.modulo = modulo
+	self._monotonic = _monotonic
 
 	self._metadata = kwargs
 
@@ -108,6 +125,65 @@ class Axis(object):
 
 	return Axis(values, self.name, weights=weights, **self._metadata)
 
+    def union(self, other):
+	""" join two Axis objects
+
+	Examples:
+	---------
+
+	>>> ax1 = Axis([0, 1, 2, 3, 4], name='myaxis')
+	>>> ax2 = Axis([-3, 2, 3, 6], name='myaxis')
+	>>> ax3 = ax1.union(ax2)
+	>>> ax3.values
+	array([-3,  0,  1,  2,  3,  4,  6])
+	"""
+	#assert isinstance(other, Axis), "can only make the Union of two Axis objects"
+	if isinstance(other, Axis):
+	    assert self.name == other.name, "axes have different names, cannot make union"
+	else:
+	    other = Axis(other, self.name) # to give it the same methods is_monotonic etc...
+	
+	if np.all(self.values == other.values):
+	    # TODO: check other attributes such as weights
+	    return self.copy()
+
+	joined = np.unique(np.concatenate([self.values, [val for val in other.values if val not in self.values]]))
+
+	# join two sorted axes?
+	if self.is_monotonic() and other.is_monotonic():
+	    is_increasing = lambda ax: ax.size < 2 or ax.values[1] >= ax.values[0]
+	    is_decreasing = lambda ax: ax.size < 2 or ax.values[1] <= ax.values[0]
+
+	    if is_increasing(self) and is_increasing(other):
+		joined.sort()
+
+	    elif is_decreasing(self) and is_decreasing(other):
+		joined2 = joined[::-1] # reverse should be increasing
+		joined2.sort()
+		joined = joined2[::-1] # back to normal
+
+	    # one increases, the other decreases !
+	    else:
+		joined.sort()
+
+	return Axis(joined, self.name)
+
+    def is_monotonic(self):
+	""" return True if monotonic
+	"""
+	if self._monotonic is None:
+	    self._monotonic = is_monotonic(self.values)
+
+	return self._monotonic
+
+    def is_regular(self):
+	""" return True if regular axis (numeric and steadily increasing)
+	"""
+	if self._regular is None:
+	    self._regular = self.is_numeric() and self.is_monotonic() and is_regular(self.values)
+
+	return self._regular
+
     @property
     def weights(self):
 	return self._weights
@@ -133,19 +209,19 @@ class Axis(object):
 	""" Access the slicer to locate axis elements
 
 	>>> ax = Axis([1,2,3],'x0')
-	>>> ax.isnumeric()
+	>>> ax.is_numeric()
 	True
 	>>> ax = Axis(['a','b','c'],'x0')
-	>>> ax.isnumeric()
+	>>> ax.is_numeric()
 	False
 	"""
 	assert self.values.ndim == 1, "!!! 2-dimensional axis !!!"
-	if self.isnumeric():
+	if self.is_numeric():
 	    return NumLocator(self.values, modulo=self.modulo)
 	else:
 	    return ObjLocator(self.values)
 
-    def isnumeric(self):
+    def is_numeric(self):
 	""" numeric type?
 	"""
 	return self.values.dtype in (np.dtype(int), np.dtype(long), np.dtype(float))
@@ -237,7 +313,7 @@ class Axis(object):
 	return cls(index.values, name=index.name)
 
 class GroupedAxis(Axis):
-    """ an Axis that contains several axes flattened together
+    """ an Axis which is a grouping of several axes flattened together
     """
     modulo = None
 
@@ -248,36 +324,48 @@ class GroupedAxis(Axis):
 	"""
 	self.axes = Axes(axes)
 	self.name = ",".join([ax.name for ax in self.axes])
+	self._values = None  # values not computed unless needed
+	self._weights = None  
+	self._size = None  
 
     @property
     def values(self):
 	""" values as 2-D numpy array, to keep things consistent with Axis
 	"""
+	if self._values is None:
+	    self._values = self._get_values()
+	return self._values
+
+    def _get_values(self):
 	# Each element of the new axis is a tuple, which makes a 2-D numpy array
 	aval = _flatten(*[ax.values for ax in self.axes])
-	#list_of_tuples = zip(*val.T.tolist())
 	val = np.empty(aval.shape[0], dtype=object)
 	val[:] = zip(*aval.T.tolist()) # pass a list of tuples
 	return val 
-	#return _flatten(*[ax.values for ax in self.axes])
 
     @property
     def weights(self):
 	""" Combine the weights as a product of the individual axis weights
 	"""
+	if self._weights is None:
+	    self._weights = self._get_weights()
+	return self._weights
+
+    def _get_weights(self):
 	if np.all([ax.weights is None for ax in self.axes]):
 	    return None
-
 	else:
 	    return _flatten(*[ax.get_weights() for ax in self.axes]).prod(axis=1)
 
-	#if np.all(axis_weights == 1):
-	#    axis_weights = None
-
     @property
     def size(self): 
 	""" size as product of axis sizes
 	"""
+	if self._size is None:
+	    self._size = self._get_size()
+	return self._size
+
+    def _get_size(self):
 	return np.prod([ax.size for ax in self.axes])
 
     @property
@@ -358,7 +446,14 @@ class Axes(list):
 
     @staticmethod
     def _init(*args, **kwargs):
-	return _init_axes(*args, **kwargs)
+	# try to catch errors one level higher
+	try:
+	    axes = _init_axes(*args, **kwargs)
+	except TypeError, msg:
+	    raise TypeError(msg)
+	except ValueError, msg:
+	    raise ValueError(msg)
+	return axes
 
     @classmethod
     def from_tuples(cls, *tuples_name_values):
@@ -366,7 +461,7 @@ class Axes(list):
 
 	Axes.from_tuples(('lat',mylat), ('lon',mylon)) 
 	"""
-	assert type(tuples_name_values[0]) is tuple, "need to provide a list of `name, values` tuples !"
+	assert np.all([type(tup) is tuple for tup in tuples_name_values]), "need to provide a list of `name, values` tuples !"
 
 	newaxes = cls()
 	for nm, values in tuples_name_values:
@@ -549,20 +644,19 @@ def _init_axes(axes=None, dims=None, labels=None, shape=None, raise_warning=True
 	return Axes()
 
     # list of Axis objects
-    elif isinstance(axes[0], Axis):
+    elif np.all([isinstance(ax, Axis) for ax in axes]):
 	axes = Axes(axes)
 
     # (name, values) tuples
-    elif isinstance(axes[0], tuple):
+    elif np.all([isinstance(ax, tuple) for ax in axes]):
 	axes = Axes.from_tuples(*axes)
 
     # axes contains only axis values, with names possibly provided in `dims=`
-    #elif is_array_equiv(axes[0]): 
-    elif type(axes[0]) in (list, np.ndarray): 
+    elif np.all([type(ax) in (list, np.ndarray) for ax in axes]):
 	axes = Axes.from_arrays(axes, dims=dims)
 
     # axes only cointain axis labels
-    elif type(axes[0]) in (str, unicode):
+    elif np.all([type(ax) in (str, unicode) for ax in axes]):
 	axes = Axes.from_shape(shape, dims=axes)
 
     else:
@@ -570,6 +664,8 @@ def _init_axes(axes=None, dims=None, labels=None, shape=None, raise_warning=True
 
     return axes
 
+
+
 #
 # Locate values on an axis
 #
