@@ -10,10 +10,10 @@ from dimarray.config import Config
 from metadata import MetadataDesc
 from axes import Axis, Axes, GroupedAxis
 
-import _transform  # numpy along-axis transformations, interpolation
-import _reshape	   # change array shape and dimensions
-import _indexing   # perform slicing and indexing operations
-import _operation  # operation between DimArrays
+import transform as _transform  # numpy along-axis transformations, interpolation
+import reshape as _reshape      # change array shape and dimensions
+import indexing as _indexing    # perform slicing and indexing operations
+import operation as _operation  # operation between DimArrays
 import missingvalues # operation between DimArrays
 
 from tools import pandas_obj
@@ -632,8 +632,8 @@ mismatch between values and axes""".format(inferred, self.values.shape)
     #
     # change `arg` to `loc`, suggesting change in indexing behaviour
     #
-    locmin = _transform.locmin
-    locmax = _transform.locmax
+    argmin = _transform.argmin
+    argmax = _transform.argmax
 
     cumsum = _transform.cumsum
     cumprod = _transform.cumprod
@@ -642,16 +642,16 @@ mismatch between values and axes""".format(inferred, self.values.shape)
     #
     # METHODS TO CHANGE ARRAY SHAPE AND SIZE
     #
-    #from _reshape import repeat, newaxis, transpose, reshape, broadcast, group, ungroup, squeeze
     repeat = _reshape.repeat
     newaxis = _reshape.newaxis
     squeeze = _reshape.squeeze
+    group = _reshape.group
+    flatten = _reshape.flatten
+    ungroup = _reshape.ungroup
+    reshape = _reshape.reshape
     transpose = _reshape.transpose
     swapaxes = _reshape.swapaxes
-    reshape = _reshape.reshape
     broadcast = _reshape.broadcast
-    group = _reshape.group
-    ungroup = _reshape.ungroup
     groupby = _reshape.groupby
     
     @property
@@ -874,7 +874,7 @@ mismatch between values and axes""".format(inferred, self.values.shape)
 	return self.values.__contains__(value)
 
     @classmethod
-    def from_arrays(cls, data, keys=None, axis="items"):
+    def from_arrays(cls, data, keys=None, axis=None):
 	""" initialize a DimArray from a dictionary of smaller dimensional DimArray
 
 	Convenience method for: collect.Dataset(data, keys).to_array(axis)
@@ -882,7 +882,7 @@ mismatch between values and axes""".format(inferred, self.values.shape)
 	input:
 	    - data : list or dict of DimArrays
 	    - keys, optional : labels of the first dimension (if dict, only useful for ordering)
-	    - axis, optional : dimension name along which to aggregate data (default "items")
+	    - axis, optional : dimension name along which to aggregate data (default "unnamed")
 
 	output:
 	    - new DimArray object, with axis alignment (reindexing)
@@ -894,15 +894,15 @@ mismatch between values and axes""".format(inferred, self.values.shape)
 	>>> a = DimArray([1,2,3])
 	>>> DimArray.from_arrays([a, 2*a]) # if keys not provided, name is 'v0', 'v1'
 	dimarray: 6 non-null elements (0 null)
-	dimensions: 'items', 'x0'
-	0 / items (2): v0 to v1
+	dimensions: 'unnamed', 'x0'
+	0 / unnamed (2): v0 to v1
 	1 / x0 (3): 0 to 2
 	array([[1, 2, 3],
 	       [2, 4, 6]])
 
-	From a dict, here also with axis alignment
+	From a dict, here also with axis alignment, and naming the axis
 	>>> d = {'a':DimArray([10,20,30.],[0,1,2]), 'b':DimArray([1,2,3.],[1.,2.,3])}
-	>>> a = DimArray.from_arrays(d, keys=['a','b']) # keys= just needed to enforce ordering
+	>>> a = DimArray.from_arrays(d, keys=['a','b'], axis='items') # keys= just needed to enforce ordering
 	>>> a
 	dimarray: 6 non-null elements (2 null)
 	dimensions: 'items', 'x0'
@@ -1073,10 +1073,14 @@ mismatch between values and axes""".format(inferred, self.values.shape)
     def __repr__(self):
 	""" pretty printing
 	"""
-	if self.ndim > 0:
-	    nonnull = np.size(self.values[~np.isnan(self.values)])
-	else:
-	    nonnull = ~np.isnan(self.values)
+	try:
+	    if self.ndim > 0:
+		nonnull = np.size(self.values[~np.isnan(self.values)])
+	    else:
+		nonnull = ~np.isnan(self.values)
+
+	except TypeError: # e.g. object
+	    nonnull = self.size
 
 	lines = []
 
@@ -1164,6 +1168,13 @@ def array_kw(*args, **kwargs):
 
 array_kw.__doc__ = DimArray.from_kw.__doc__
 
+# 
+# EXPERIMENTAL: STILL THINKING ABOUT WHAT IS THE BEST SYNTAX
+# array
+# join
+# from_arrays
+# 
+
 def array(data, *args, **kwargs):
     """ wrapper for DimArray and DimArray.from_arrays
 
@@ -1172,7 +1183,6 @@ def array(data, *args, **kwargs):
 
     *args, **kwargs: variable argument for DimArray or DimArray.from_arrays
 
-
     See also:
     DimArray, DimArray.from_arrays
     """
@@ -1189,9 +1199,10 @@ def array(data, *args, **kwargs):
 join = DimArray.from_arrays
 
 # handy aliases
-def from_arrays(*args, **kwargs):
-    warnings.warn(FutureWarning('from_arrays is deprecated, use da.array() instead'))
-    return DimArray.from_arrays(*args, **kwargs) 
+#def from_arrays(*args, **kwargs):
+#    warnings.warn(FutureWarning('from_arrays is deprecated, use da.array() instead'))
+#    return DimArray.from_arrays(*args, **kwargs) 
+from_arrays = DimArray.from_arrays
 
 from_pandas = DimArray.from_pandas
 
